package com.usky.dxtop.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sun.jna.platform.win32.Winspool;
import com.sun.org.apache.xpath.internal.operations.Bool;
import com.usky.dxtop.common.constant.UserConstants;
import com.usky.dxtop.common.core.domain.model.LoginUser;
import com.usky.dxtop.common.core.page.CommonPage;
import com.usky.dxtop.common.exception.CustomException;
import com.usky.dxtop.common.utils.*;
import com.usky.dxtop.model.*;
import com.usky.dxtop.mapper.ProductInfoMapper;
import com.usky.dxtop.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.usky.dxtop.service.emun.AsyncResultType;
import com.usky.dxtop.service.vo.OrderExport;
import com.usky.dxtop.service.vo.OrderRequest;
import com.usky.dxtop.service.vo.ProductInfoExport;
import com.usky.dxtop.service.vo.ProductInfoRequest;
import jdk.nashorn.internal.ir.LiteralNode;
import org.apache.commons.math3.stat.descriptive.summary.Product;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.aop.framework.AopContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.security.core.parameters.P;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.lang.reflect.Array;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 商品详情 服务实现类
 * </p>
 *
 * @author yq
 * @since 2022-07-05
 */
@Service
public class ProductInfoServiceImpl extends ServiceImpl<ProductInfoMapper, ProductInfo> implements ProductInfoService {

    @Autowired
    private ProductTypeService productTypeService;

    @Autowired
    private ISysAsyncTaskService sysAsyncTaskService;

    @Autowired
    private SysFileService sysFileService;

    @Autowired
    private ProductPeriodService productPeriodService;


    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean add(ProductInfo productInfo) {
        if (UserConstants.NOT_UNIQUE.equals(this.checkNameUnique(productInfo)))
        {
            throw new CustomException("新增商品信息'" + productInfo.getProductName() + "'失败，商品名称已存在");
        }
        checkTimeBetween(productInfo);
        this.save(productInfo);
        productPeriodService.insertList(productInfo);
        return true;
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(ProductInfo productInfo) {
        if (UserConstants.NOT_UNIQUE.equals(this.checkNameUnique(productInfo)))
        {
            throw new CustomException("修改商品信息'" + productInfo.getProductName() + "'失败，商品名称已存在");
        }
        checkTimeBetween(productInfo);
        productPeriodService.removeByProductInfo(productInfo.getId());
        productPeriodService.insertList(productInfo);
        return this.updateById(productInfo);
    }

    @Override
    public String checkNameUnique(ProductInfo productInfo) {
        Long id = StringUtils.isNull(productInfo.getId()) ? -1L : productInfo.getId();
        LambdaQueryWrapper<ProductInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ProductInfo::getProductName,productInfo.getProductName())
                .eq(ProductInfo::getProductTypeId,productInfo.getProductTypeId());
        ProductInfo one = this.getOne(queryWrapper);
        if (StringUtils.isNotNull(one) && one.getId() != id.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }


    public void checkTimeBetween(ProductInfo productInfo){
        if (baseMapper.checkTime(productInfo) > 0){
            throw new CustomException("选择的时间和周期有重复,重新选择");
        }

    }

    @Override
    public CommonPage<ProductInfo> list(ProductInfoRequest productInfoRequest) {
        IPage<ProductInfo> page = new Page<>(productInfoRequest.getCurrent(), productInfoRequest.getSize());
        baseMapper.page(page,productInfoRequest);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }

    @Override
    public String export(ProductInfoRequest productInfoRequest) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        SysUser user = loginUser.getUser();
        Long taskId = sysAsyncTaskService.getId(user.getUserName(), "订单详情");
        if (null == taskId) {
            throw new CustomException("获取次数太多了请稍后获取");
        }
        ((ProductInfoServiceImpl) AopContext.currentProxy()).syncInfo(taskId,productInfoRequest);
        return String.valueOf(taskId);
    }


    @Async
    public void syncInfo(Long taskId, ProductInfoRequest productInfoRequest){
        Workbook workbook = null;
        try {
            ExportParams params = new ExportParams(null, "商品详情");
            workbook = ExcelExportUtil.exportBigExcel(params, ProductInfoExport.class,
                    (o, i) -> {
                        productInfoRequest.setCurrent(i);
                        productInfoRequest.setSize(30);
                        CommonPage<ProductInfo> page = this.list(productInfoRequest);
                        return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), ProductInfo.class, ProductInfoExport.class));
                    },null);
            if (null != workbook) {
                String fileName = String.format("%s.xls", "商品详情");
                MultipartFile multipartFile = ExcelUtils.downLoadExcel(workbook, fileName);
                uploadOrderFile(multipartFile,taskId);
            } else {
                sysAsyncTaskService.sendResult(taskId, false, "商品详情表格数据为空", AsyncResultType.TEXT);
            }
        } catch (Exception e) {
            log.error("导出文件失败", e);
            sysAsyncTaskService.sendResult(taskId, false, "导出订单文件失败", AsyncResultType.TEXT);
        } finally {
            if (workbook != null) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    log.error("===export spec=== 关闭workbook失败", e);
                }
            }
        }
    }

    public void uploadOrderFile(MultipartFile multipartFile, Long taskId){
        List<MultipartFile> list = new ArrayList<>();
        list.add(multipartFile);
        String batchNo = sysFileService.generateBatchNo();
        SysFile sysFile = new SysFile();
        sysFile.setBatchNo(batchNo);
        sysFile.setFiles(list);
        sysFile.setUploadType(SysFileServiceImpl.FILE);
        sysFile.setBusinessType("商品详情");
        sysFile.setExpriceAt(DateUtils.addDays(new Date(), 3));
        List<SysFile> sysFileList = sysFileService.addFile(sysFile);
        long count = sysFileList.stream().filter(sysFileDTO -> !sysFileDTO.getSuccess()).count();
        if (count > 0) {
            sysAsyncTaskService.sendResult(taskId, false, "上传文件失败", AsyncResultType.TEXT);
            return;
        }
        List<String> idList = sysFileList.stream().map(sf -> sf.getId().toString()).collect(Collectors.toList());
        boolean active = sysFileService.active(batchNo,idList);
        if (active) {
            sysAsyncTaskService.sendResult(taskId, true, batchNo, AsyncResultType.FILE);
        } else {
            sysAsyncTaskService.sendResult(taskId, false, "上传文件失败", AsyncResultType.TEXT);
        }
    }





    public List<ProductType> getByIds(List<Long> produceTypeId){
        LambdaQueryWrapper<ProductType> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .select(ProductType::getId,ProductType::getName)
                .in(ProductType::getId,produceTypeId);
        return productTypeService.list(queryWrapper);
    }

    public void enhanceDate(ProductInfo productInfo,ProductType productType){
        productInfo.setProductTypeName(productType.getName());
    }


    @Override
    public boolean remove(Long id) {
        return false;
    }
}
