package com.usky.dxtop.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.dxtop.common.constant.UserConstants;
import com.usky.dxtop.common.core.page.CommonPage;
import com.usky.dxtop.common.exception.CustomException;
import com.usky.dxtop.common.utils.StringUtils;
import com.usky.dxtop.model.Company;
import com.usky.dxtop.model.ProductType;
import com.usky.dxtop.mapper.ProductTypeMapper;
import com.usky.dxtop.service.ProductTypeService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 商品详情 服务实现类
 * </p>
 *
 * @author yq
 * @since 2022-07-05
 */
@Service
public class ProductTypeServiceImpl extends ServiceImpl<ProductTypeMapper, ProductType> implements ProductTypeService {


    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean add(ProductType productType) {
        if (UserConstants.NOT_UNIQUE.equals(this.checkNameUnique(productType)))
        {
            throw new CustomException("新增商品类别信息'" + productType.getName() + "'失败，商品类别已存在");
        }
        return this.save(productType);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(ProductType productType) {
        if (UserConstants.NOT_UNIQUE.equals(this.checkNameUnique(productType)))
        {
            throw new CustomException("修改商品类别信息'" + productType.getName() + "'失败，商品类别已存在");
        }
        return this.updateById(productType);
    }

    @Override
    public String checkNameUnique(ProductType productType) {
        Long id = StringUtils.isNull(productType.getId()) ? -1L : productType.getId();
        LambdaQueryWrapper<ProductType> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ProductType::getName,productType.getName());
        ProductType one = this.getOne(queryWrapper);
        if (StringUtils.isNotNull(one) && one.getId() != id.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    @Override
    public CommonPage<ProductType> list(Integer current, Integer size, String name, Boolean isTake) {
        IPage<ProductType> page = new Page<>(current, size);
        LambdaQueryWrapper<ProductType> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.like(StringUtils.isNotBlank(name),ProductType::getName,name)
                .eq(null != isTake,ProductType::getIsTake,isTake)
                .orderByDesc(ProductType::getId);
        page = this.page(page,queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }

    @Override
    public boolean remove(Long id) {
        return false;
    }
}
